﻿using System;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.EJ.ReportViewer;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.AspNetCore.Hosting;
using System.IO;
using Syncfusion.JavaScript.Models.ReportViewer;
using Newtonsoft.Json;
using Syncfusion.RDL.DOM;

namespace ReportViewerCore.API
{
    public partial class ReportApiController : Controller, IReportController
    {
        private IMemoryCache _cache;
        private IHostingEnvironment _hostingEnvironment;

        public ReportApiController(IMemoryCache memoryCache, IHostingEnvironment hostingEnvironment)
        {
            _cache = memoryCache;
            _hostingEnvironment = hostingEnvironment;
        }

        [HttpPost]
        public object PostReportAction([FromBody] Dictionary<string, object> jsonResult)
        {
            return ReportHelper.ProcessReport(jsonResult, this, this._cache);
        }

        [ActionName("GetResource")]
        [AcceptVerbs("GET")]
        public object GetResource(ReportResource resource)
        {
            return ReportHelper.GetResource(resource, this, _cache);
        }

        [HttpPost]
        public object PostFormReportAction()
        {
            return ReportHelper.ProcessReport(null, this, this._cache);
        }

        public void OnInitReportOptions(ReportViewerOptions reportOption)
        {
            ReportDefinition report = new ReportDefinition();
            report.ReportSections = new ReportSections();
            var reportSection = new ReportSection();
            report.ReportSections.Add(reportSection);
            reportSection.Body = CreateBody();
            reportSection.Page = new Syncfusion.RDL.DOM.Page();
            reportSection.Page.PageHeader = CreateHeader();
            reportSection.Page.PageFooter = CreateFooter();
            reportSection.Page.Style = AddStyle();
            reportSection.Page.PageHeight = "4in";
            reportSection.Page.PageWidth = "6in";
            reportSection.Width = new Syncfusion.RDL.DOM.Size("6in");
            report.ReportUnitType = "Inch";
            report.RDLType = RDLType.RDL2010;


            //string basePath = _hostingEnvironment.WebRootPath;
            //FileStream reportStream = new FileStream(basePath + @"\ReportRDL\GroupingAgg.rdl", FileMode.Open, FileAccess.Read);
            reportOption.ReportModel.ReportDefinition = report;
        }

        Syncfusion.RDL.DOM.Style AddStyle()
        {
            Syncfusion.RDL.DOM.Style style = new Syncfusion.RDL.DOM.Style();
            style.Border = new Syncfusion.RDL.DOM.Border();
            style.Border.Width = new Syncfusion.RDL.DOM.Size("1pt");
            style.Border.Style = "Solid";
            style.Border.Color = "Black";
            return style;
        }

        PageHeader CreateHeader()
        {
            PageHeader pageHeader = new PageHeader();
            pageHeader.Height = new Syncfusion.RDL.DOM.Size("0.59167in");
            pageHeader.Style = AddStyle();
            pageHeader.PrintOnFirstPage = true;
            pageHeader.PrintOnLastPage = true;
            pageHeader.ReportItems = new ReportItems();
            var textBox = CreateTextBox("TextBox2", "Header", "1.61333in", "0.13833in", "2.2in", "0.34167in");
            pageHeader.ReportItems.Add(textBox);
            return pageHeader;
        }

        PageFooter CreateFooter()
        {
            PageFooter pageFooter = new PageFooter();
            pageFooter.Style = AddStyle();
            pageFooter.Height = new Syncfusion.RDL.DOM.Size("0.59167in");
            pageFooter.ReportItems = new ReportItems();
            pageFooter.PrintOnFirstPage = true;
            pageFooter.PrintOnLastPage = true;
            var textBox = CreateTextBox("TextBox3", "Footer", "1.61333in", "0.05416in", "2.2in", "0.34167in");
            pageFooter.ReportItems.Add(textBox);
            return pageFooter;
        }

        Body CreateBody()
        {
            var body = new Body();
            body.Height = new Syncfusion.RDL.DOM.Size("2.03333in");
            body.Style = AddStyle();
            body.ReportItems = new ReportItems();
            var textBox = CreateTextBox("TextBox1", "Body", "1.58833in", "0.66333in", "2.225in", "0.59167in");
            body.ReportItems.Add(textBox);
            return body;
        }

        Syncfusion.RDL.DOM.TextBox CreateTextBox(string name, string text, string left, string top, string width, string height)
        {
            var textBox = new Syncfusion.RDL.DOM.TextBox();
            textBox.Name = name;
            textBox.Height = new Syncfusion.RDL.DOM.Size(height);
            textBox.Width = new Syncfusion.RDL.DOM.Size(width);
            textBox.Left = new Syncfusion.RDL.DOM.Size(left);
            textBox.Top = new Syncfusion.RDL.DOM.Size(top);
            textBox.Style = new Syncfusion.RDL.DOM.Style();
            textBox.Style.Border = new Syncfusion.RDL.DOM.Border();
            textBox.Style.Border.Width = new Syncfusion.RDL.DOM.Size("1pt");
            textBox.Style.Border.Style = "Solid";
            textBox.Style.Border.Color = "Black";
            textBox.Style.TextAlign = "Center";
            textBox.Style.VerticalAlign = "Top";

            textBox.Paragraphs = new Paragraphs();
            Syncfusion.RDL.DOM.Paragraph paragraph = new Syncfusion.RDL.DOM.Paragraph();
            TextRuns runs = new TextRuns();
            TextRun run = new TextRun();
            run.Style = new Syncfusion.RDL.DOM.Style();
            run.Style.FontStyle = "Default";
            run.Style.TextAlign = "Center";
            run.Style.FontFamily = "Arial";
            run.Style.FontSize = new Syncfusion.RDL.DOM.Size("10pt");
            run.Value = text;
            runs.Add(run);
            paragraph.Style = new Syncfusion.RDL.DOM.Style();
            paragraph.Style.VerticalAlign = "Top";
            paragraph.Style.TextAlign = "Center";
            paragraph.TextRuns = runs;
            textBox.Paragraphs.Add(paragraph);



            return textBox;
        }

        public void OnReportLoaded(ReportViewerOptions reportOption)
        {

        }
    }
}
